#!/usr/bin/python
"""
Filters out some of the #defines used throughout the GCC sources:
- GTY(()) marks declarations for gengtype.c
- PARAMS(()) is used for K&R compatibility. See ansidecl.h.

When passed one or more filenames, acts on those files and prints the
results to stdout.

When run without a filename, runs a unit-testing suite.
"""
import re
import sys
import unittest

# Optional whitespace
OPT_WS = '\s*'

def filter_src(text):
    """
    str -> str.  We operate on the whole of the source file at once
    (rather than individual lines) so that we can have multiline
    regexes.
    """

    # Convert C comments from GNU coding convention of:
    #    /* FIRST_LINE
    #       NEXT_LINE
    #       FINAL_LINE.  */
    # to:
    #    /** @verbatim FIRST_LINE
    #       NEXT_LINE
    #       FINAL_LINE.  @endverbatim */
    # so that doxygen will parse them.
    #
    # Only comments that begin on the left-most column are converted.
    #
    text = re.sub(r'^/\*\* ',
                  r'/** @verbatim ',
                  text,
                  flags=re.MULTILINE)
    text = re.sub(r'^/\* ',
                  r'/** @verbatim ',
                  text,
                  flags=re.MULTILINE)
    text = re.sub(r'\*/',
                  r' @endverbatim */',
                  text)

    # Remove GTY markings (potentially multiline ones):
    text = re.sub('GTY' + OPT_WS + r'\(\(.*?\)\)\s+',
                  '',
                  text,
                  flags=(re.MULTILINE|re.DOTALL))

    # Strip out 'ATTRIBUTE_UNUSED'
    text = re.sub('\sATTRIBUTE_UNUSED',
                  '',
                  text)

    # PARAMS(()) is used for K&R compatibility. See ansidecl.h.
    text = re.sub('PARAMS' + OPT_WS + r'\(\((.*?)\)\)',
                  r'(\1)',
                  text)

    # Replace 'ENUM_BITFIELD(enum_name)' with 'enum enum_name'.
    text = re.sub('ENUM_BITFIELD\s*\(([^\)]*)\)',
                  r'enum \1',
                  text)

    return text

class FilteringTests(unittest.TestCase):
    '''
    Unit tests for filter_src.
    '''
    def assert_filters_to(self, src_input, expected_result):
        # assertMultiLineEqual was added to unittest in 2.7/3.1
        if hasattr(self, 'assertMultiLineEqual'):
            assertion = self.assertMultiLineEqual
        else:
            assertion = self.assertEqual
        assertion(expected_result, filter_src(src_input))

    def test_comment_example(self):
        self.assert_filters_to(
            ('/* FIRST_LINE\n'
             '   NEXT_LINE\n'
             '   FINAL_LINE.  */\n'),
            ('/** @verbatim FIRST_LINE\n'
             '   NEXT_LINE\n'
             '   FINAL_LINE.   @endverbatim */\n'))

    def test_comment_example_gengtype(self):
        self.assert_filters_to(
            ('/** Allocate and initialize an input buffer state.\n'
             ' * @param file A readable stream.\n'
             ' * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.\n'
             ' * \n'
             ' * @return the allocated buffer state.\n'
             ' */'),
            ('/** @verbatim Allocate and initialize an input buffer state.\n'
             ' * @param file A readable stream.\n'
             ' * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.\n'
             ' * \n'
             ' * @return the allocated buffer state.\n'
             '  @endverbatim */'))

    def test_oneliner_comment(self):
        self.assert_filters_to(
            '/* Returns the string representing CLASS.  */\n',
            ('/** @verbatim Returns the string representing CLASS.   @endverbatim */\n'))

    def test_multiline_comment(self):
        self.assert_filters_to(
            ('/* The thread-local storage model associated with a given VAR_DECL\n'
             "   or SYMBOL_REF.  This isn't used much, but both trees and RTL refer\n"
             "   to it, so it's here.  */\n"),
            ('/** @verbatim The thread-local storage model associated with a given VAR_DECL\n'
             "   or SYMBOL_REF.  This isn't used much, but both trees and RTL refer\n"
             "   to it, so it's here.   @endverbatim */\n"))

    def test_GTY(self):
        self.assert_filters_to(
            ('typedef struct GTY(()) alias_pair {\n'
             '  tree decl;\n'
             '  tree target;\n'
             '} alias_pair;\n'),
            ('typedef struct alias_pair {\n'
             '  tree decl;\n'
             '  tree target;\n'
             '} alias_pair;\n'))

    def test_multiline_GTY(self):
        # Ensure that a multiline GTY is filtered out.
        self.assert_filters_to(
            ('class GTY((desc ("%h.type"), tag ("SYMTAB_SYMBOL"),\n'
             '\t   chain_next ("%h.next"), chain_prev ("%h.previous")))\n'
             '  symtab_node_base\n'
             '{\n'),
            ('class symtab_node_base\n'
             '{\n'))

    def test_ATTRIBUTE_UNUSED(self):
        # Ensure that ATTRIBUTE_UNUSED is filtered out.
        self.assert_filters_to(
            ('static void\n'
             'record_set (rtx dest, const_rtx set, void *data ATTRIBUTE_UNUSED)\n'
             '{\n'),
            ('static void\n'
             'record_set (rtx dest, const_rtx set, void *data)\n'
             '{\n'))

    def test_PARAMS(self):
        self.assert_filters_to(
            'char *strcpy PARAMS ((char *dest, char *source));\n',
            'char *strcpy (char *dest, char *source);\n')

    def test_ENUM_BITFIELD(self):
        self.assert_filters_to(
            '  ENUM_BITFIELD (sym_intent) intent:2;\n',
            '  enum sym_intent intent:2;\n')

def act_on_files(argv):
    for filename in argv[1:]:
        with open(filename) as f:
            text = f.read()
            print(filter_src(text))

if __name__ == '__main__':
    if len(sys.argv) > 1:
        act_on_files(sys.argv)
    else:
        unittest.main()



#!/usr/bin/env python3
#
# Script to mark bunch of PRs as spam 
#
# This file is part of GCC.
#
# GCC is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# GCC is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with GCC; see the file COPYING3.  If not see
# <http://www.gnu.org/licenses/>.  */
#
#
#

import requests
import json
import argparse

base_url = 'https://gcc.gnu.org/bugzilla/rest.cgi/'

def mark_as_spam(id, api_key, verbose):
    print('Marking as spam: PR%d' % id)
    # 1) get bug info to find 'cc'
    u = base_url + 'bug/' + str(id)
    r = requests.get(u)
    response = json.loads(r.text)

    if 'error' in response and response['error']:
        print(response['message'])
        return

    # 2) mark the bug as spam
    bug = response['bugs'][0]
    creator = bug['creator']
    cc_list = bug['cc']
    data = {
        'status': 'RESOLVED',
        'resolution': 'INVALID',
        'summary': 'spam',
        'ids': [id],
        'api_key': api_key,
        'comment': { 'comment': 'spam'},
        'product': 'gcc',
        'component': 'spam',
        'version': 'unknown',
        'cc': {'remove': cc_list},
        'priority': 'P5',
        'severity': 'trivial',
        'url': '',
        'assigned_to': 'unassigned@gcc.gnu.org' }

    r = requests.put(u, json = data)
    if verbose:
        print(r)
        print(r.text)

    # 3) mark the first comment as spam
    r = requests.get(u + '/comment')
    response = json.loads(r.text)
    for c in response['bugs'][str(id)]['comments']:
        if c['creator'] == creator:
            comment_id = c['id']
            u2 = '%sbug/comment/%d/tags' % (base_url, comment_id)
            print(u2)
            r = requests.put(u2, json = {'comment_id': comment_id, 'add': ['spam'], 'api_key': api_key})
            if verbose:
                print(r)
                print(r.text)

    # 4) mark all attachments as spam
    r = requests.get(u + '/attachment')
    response = json.loads(r.text)
    attachments = response['bugs'][str(id)]
    for a in attachments:
        attachment_id = a['id']
        url = '%sbug/attachment/%d' % (base_url, attachment_id)
        r = requests.put(url, json = {'ids': [attachment_id],
            'summary': 'spam',
            'file_name': 'spam',
            'content_type': 'application/x-spam',
            'is_obsolete': True,
            'api_key': api_key})
        if verbose:
            print(r)
            print(r.text)

parser = argparse.ArgumentParser(description='Mark Bugzilla issues as spam.')
parser.add_argument('api_key', help = 'API key')
parser.add_argument('range', help = 'Range of IDs, e.g. 10-23,24,25,27')
parser.add_argument('--verbose', action = 'store_true', help = 'Verbose logging')

args = parser.parse_args()

chunks = args.range.split(',')
for c in chunks:
    parts = list(map(lambda x: int(x), c.split('-')))
    if len(parts) == 1:
        r = [parts[0]]
    else:
        r = range(parts[0], parts[1] + 1)

    for id in r:
        mark_as_spam(id, args.api_key, args.verbose)
